from linux_router import LinuxRouter
from mininet.topo import Topo
from topologies.custom_topo import CustomTopo


class SixRoutersFourHosts(CustomTopo):
    def build(self):
        # we pre defined count of hosts for cpu limitations
        self.host_count = 10
        # Adding routers
        R1 = self.addNode('r1', cls=LinuxRouter, ip='10.1.0.1/24')
        R2 = self.addNode('r2', cls=LinuxRouter, ip='10.2.0.1/24')
        R3 = self.addNode('r3', cls=LinuxRouter, ip='10.3.0.1/24')
        R4 = self.addNode('r4', cls=LinuxRouter, ip='10.4.0.1/24')
        R5 = self.addNode('r5', cls=LinuxRouter, ip='10.5.0.1/24')
        R6 = self.addNode('r6', cls=LinuxRouter, ip='10.6.0.1/24')

        # Adding switches
        S1 = self.addSwitch('s1')
        S2 = self.addSwitch('s2')
        S3 = self.addSwitch('s3')
        S4 = self.addSwitch('s4')
        S5 = self.addSwitch('s5')
        S6 = self.addSwitch('s6')

        # Connecting switches to routers
        self.addLink(S1, R1, intfName2='r1-eth0', params2={'ip': '10.1.0.1/24'})
        self.addLink(S2, R2, intfName2='r2-eth0', params2={'ip': '10.2.0.1/24'})
        self.addLink(S3, R3, intfName2='r3-eth0', params2={'ip': '10.3.0.1/24'})
        self.addLink(S4, R4, intfName2='r4-eth0', params2={'ip': '10.4.0.1/24'})
        self.addLink(S5, R5, intfName2='r5-eth0', params2={'ip': '10.5.0.1/24'})
        self.addLink(S6, R6, intfName2='r6-eth0', params2={'ip': '10.6.0.1/24'})

        # Connecting routers to each other
        self.addLink(R1, R2, intfName1='r1-eth1', intfName2='r2-eth1', params1={'ip': '10.112.0.11/24'},
                     params2={'ip': '10.112.0.21/24'})
        self.addLink(R2, R3, intfName1='r2-eth2', intfName2='r3-eth1', params1={'ip': '10.123.0.22/24'},
                     params2={'ip': '10.123.0.31/24'})
        self.addLink(R1, R4, intfName1='r1-eth2', intfName2='r4-eth1', params1={'ip': '10.114.0.12/24'},
                     params2={'ip': '10.114.0.41/24'})
        self.addLink(R4, R5, intfName1='r4-eth2', intfName2='r5-eth1', params1={'ip': '10.145.0.42/24'},
                     params2={'ip': '10.145.0.51/24'})
        self.addLink(R3, R6, intfName1='r3-eth2', intfName2='r6-eth1', params1={'ip': '10.136.0.32/24'},
                     params2={'ip': '10.136.0.61/24'})
        self.addLink(R5, R6, intfName1='r5-eth2', intfName2='r6-eth2', params1={'ip': '10.156.0.52/24'},
                     params2={'ip': '10.156.0.62/24'})

        H1 = self.addHost('h1', ip='10.1.0.101/24', defaultRoute='via 10.1.0.1')
        H3 = self.addHost('h3', ip='10.3.0.101/24', defaultRoute='via 10.3.0.1')
        H4 = self.addHost('h4', ip='10.4.0.101/24', defaultRoute='via 10.4.0.1')
        H6 = self.addHost('h6', ip='10.6.0.101/24', defaultRoute='via 10.6.0.1')

        for h, s in [(H1, S1), (H3, S3), (H4, S4), (H6, S6)]:
            self.addLink(h, s)

    def get_routings(self):
        return {
            "r1": {
                "default": [
                    ("10.2.0.0/24", "10.112.0.21", "r1-eth1", 10),
                    ("10.3.0.0/24", "10.112.0.21", "r1-eth1", 10),
                    ("10.4.0.0/24", "10.114.0.41", "r1-eth2", 10),
                    ("10.5.0.0/24", "10.114.0.41", "r1-eth2", 10),
                    ("10.6.0.0/24", "10.112.0.21", "r1-eth1", 10),
                    ("10.6.0.0/24", "10.114.0.41", "r1-eth2", 20),

                    ("10.123.0.0/24", "10.112.0.21", "r1-eth1", 10),
                    ("10.136.0.0/24", "10.112.0.21", "r1-eth1", 10),
                    ("10.145.0.0/24", "10.114.0.41", "r1-eth2", 10),
                    ("10.156.0.0/24", "10.114.0.41", "r1-eth2", 10),
                ],
                "incoming_eth1": [
                    ("10.6.0.0/24", "10.114.0.41", "r1-eth2", 10),
                ]
            },
            "r2": {
                "default": [
                    ("10.1.0.0/24", "10.112.0.11", "r2-eth1", 10),
                    ("10.3.0.0/24", "10.123.0.31", "r2-eth2", 10),
                    ("10.4.0.0/24", "10.112.0.11", "r2-eth1", 10),
                    ("10.5.0.0/24", "10.123.0.31", "r2-eth2", 10),
                    ("10.6.0.0/24", "10.123.0.31", "r2-eth2", 10),
                    ("10.6.0.0/24", "10.112.0.11", "r2-eth1", 20),

                    ("10.136.0.0/24", "10.123.0.31", "r2-eth2", 10),
                    ("10.145.0.0/24", "10.123.0.31", "r2-eth2", 10),
                    ("10.156.0.0/24", "10.123.0.31", "r2-eth2", 10),
                    ("10.114.0.0/24", "10.112.0.11", "r2-eth1", 10),
                ],
                "incoming_eth2": [
                    ("10.6.0.0/24", "10.112.0.11", "r2-eth1", 10)
                ]
            },
            "r3": {
                "default": [
                    ("10.1.0.0/24", "10.123.0.22", "r3-eth1", 10),
                    ("10.2.0.0/24", "10.123.0.22", "r3-eth1", 10),
                    ("10.4.0.0/24", "10.123.0.22", "r3-eth1", 10),
                    ("10.5.0.0/24", "10.136.0.61", "r3-eth2", 10),
                    ("10.6.0.0/24", "10.136.0.61", "r3-eth2", 10),
                    ("10.6.0.0/24", "10.123.0.22", "r3-eth1", 20),

                    ("10.112.0.0/24", "10.123.0.22", "r3-eth1", 10),
                    ("10.145.0.0/24", "10.136.0.61", "r3-eth2", 10),
                    ("10.156.0.0/24", "10.136.0.61", "r3-eth2", 10),
                    ("10.114.0.0/24", "10.123.0.22", "r3-eth1", 10),
                ]
            },
            "r4": {
                "default": [
                    ("10.1.0.0/24", "10.114.0.12", "r4-eth1", 10),
                    ("10.2.0.0/24", "10.114.0.12", "r4-eth1", 10),
                    ("10.3.0.0/24", "10.114.0.12", "r4-eth1", 10),
                    ("10.5.0.0/24", "10.145.0.51", "r4-eth2", 10),
                    ("10.6.0.0/24", "10.145.0.51", "r4-eth2", 10),

                    ("10.112.0.0/24", "10.114.0.12", "r4-eth1", 10),
                    ("10.123.0.0/24", "10.114.0.12", "r4-eth1", 10),
                    ("10.156.0.0/24", "10.145.0.51", "r4-eth2", 10),
                    ("10.136.0.0/24", "10.145.0.51", "r4-eth2", 10),
                ]
            },
            "r5": {
                "default": [
                    ("10.1.0.0/24", "10.145.0.42", "r5-eth1", 10),
                    ("10.2.0.0/24", "10.145.0.42", "r5-eth1", 10),
                    ("10.3.0.0/24", "10.156.0.62", "r5-eth2", 10),
                    ("10.4.0.0/24", "10.145.0.42", "r5-eth1", 10),
                    ("10.6.0.0/24", "10.156.0.62", "r5-eth2", 10),

                    ("10.112.0.0/24", "10.145.0.42", "r5-eth1", 10),
                    ("10.123.0.0/24", "10.156.0.62", "r5-eth2", 10),
                    ("10.114.0.0/24", "10.145.0.42", "r5-eth1", 10),
                    ("10.136.0.0/24", "10.156.0.62", "r5-eth2", 10),
                ]
            },
            "r6": {
                "default": [
                    ("10.1.0.0/24", "10.136.0.32", "r6-eth1", 10),
                    ("10.1.0.0/24", "10.156.0.52", "r6-eth2", 20),
                    ("10.2.0.0/24", "10.136.0.32", "r6-eth1", 10),
                    ("10.3.0.0/24", "10.136.0.32", "r6-eth1", 10),
                    ("10.4.0.0/24", "10.156.0.52", "r6-eth2", 10),
                    ("10.5.0.0/24", "10.156.0.52", "r6-eth2", 10),

                    ("10.112.0.0/24", "10.136.0.32", "r6-eth1", 10),
                    ("10.123.0.0/24", "10.136.0.32", "r6-eth1", 10),
                    ("10.114.0.0/24", "10.156.0.52", "r6-eth2", 10),
                    ("10.145.0.0/24", "10.156.0.52", "r6-eth2", 10),
                ]
            }
        }

    def get_policies(self):
        return {
            "r1": {
                "rules": [
                    {
                        "table": "incoming_eth1",
                        "inport": "r1-eth1",
                        "to": ["10.6.0.0/24"]
                    },
                ]
            },
            "r2": {
                "rules": [
                    {
                        "table": "incoming_eth2",
                        "inport": "r2-eth2",
                        "to": ["10.6.0.0/24"]
                    }
                ]
            }

        }

    def get_tests(self):
        return {
            "tcp_bandwidth_intermediate": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.iperf((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "separate_definitions": True,
                    "command_pre": ("measure_bandwidth", (['h1', 'h6'], '10.6.0.101', 30, 0.5, "tcp_pre_failure", "TCP Bandwidth before failure", "tcp", 100, [0, self.limit+self.limit*0.1])),
                    "command_post": ("measure_bandwidth", (['h1', 'h6'], '10.6.0.101', 30, 0.5, "tcp_post_failure", "TCP Bandwidth after failure", "tcp", 100, [0, self.limit+self.limit*0.1]))
                },
                "failures": [
                    {
                        "type": "intermediate",
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                            "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "bandwidth_link_usage_intermediate": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.iperf((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "separate_definitions": True,
                    "command_pre": ("measure_link_usage_bandwidth", (
                        {"client": "h1", "server": "h6", "server_ip": "10.6.0.101"}, {"client": "h3", "server": "h1", "server_ip": "10.1.0.101"}, 30, 1, "bandwidth_link_usage_pre_failure", "TCP Bandwidth before failure", "tcp", 100,
                    [0, 130])),
                    "command_post": ("measure_link_usage_bandwidth", (
                    {"client": "h1", "server": "h6", "server_ip": "10.6.0.101"}, {"client": "h3", "server": "h1", "server_ip": "10.1.0.101"}, 30, 1, "bandwidth_link_usage_post_failure", "TCP Bandwidth after failure", "tcp", 100,
                    [0, 130]))
                },
                "failures": [
                    {
                        "type": "intermediate",
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "bandwidth_link_usage_concurrent": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.iperf((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "command": ("measure_link_usage_bandwidth", (
                        {"client": "h1", "server": "h6", "server_ip": "10.6.0.101"},
                        {"client": "h3", "server": "h1", "server_ip": "10.1.0.101"}, 30, 1,
                        "bandwidth_link_usage_concurrent_failure", "TCP Bandwidth with concurrent failure", "tcp", 100,
                        [0, 130])),
                },
                "failures": [
                    {
                        "type": "timer",
                        "timing": 18,
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "bandwidth_failure_concurrent": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.iperf((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "command": ("measure_bandwidth", (
                    ['h1', 'h6'], '10.6.0.101', 30, 0.5, "tcp_failure_concurrent", "TCP Bandwidth with concurrent failure",
                    "tcp", 100, [0, self.limit + self.limit * 0.1])),

                },
                "failures": [
                    {
                        "type": "timer",
                        "timing": 15,
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "latency_failure_concurrent": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.ping((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "command": ("measure_latency", (
                    'h1', '10.6.0.101', 30, 0.5, "s_to_d_latency_concurrent", [0, 150], "Latency from H1 to H6 with concurrent failure"))

                },
                "failures": [
                    {
                        "type": "timer",
                        "timing": 15,
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "latency_failure_intermediate": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.ping((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "separate_definitions": True,
                    "command_pre": ("measure_latency", (
                        'h1', '10.6.0.101', 30, 0.5, "before_failure", [0, 150], "Latency from H1 to H6 before failure")),
                    "command_post": ("measure_latency", (
                        'h1', '10.6.0.101', 30, 0.5, "after_failure", [0, 150], "Latency from H1 to H6 after failure"))

                },
                "failures": [
                    {
                        "type": "intermediate",
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "tcp_packet_flow_intermediate": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.iperf((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "separate_definitions": True,
                    "command_pre": ("measure_packet_flow", (
                        'h1', 'h6', '10.6.0.101', ["r1", "r2", "r3", "r5"], 30, 1, "before_failure", [0, 3000], "Packet flow on routers before failure", "tcp", 100)),
                    "command_post": ("measure_packet_flow", (
                        'h1', 'h6', '10.6.0.101', ["r1", "r2", "r3", "r5"], 30, 1, "after_failure", [0, 3000], "Packet flow on routers after failure", "tcp", 100)),

                },
                "failures": [
                    {
                        "type": "intermediate",
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "tcp_packet_flow_concurrent": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.iperf((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "command": ("measure_packet_flow", (
                        'h1', 'h6', '10.6.0.101', ["r1", "r2", "r3", "r5"], 30, 1, "concurrent_failure", [0, 3000], "Packet flow on routers before failure", "tcp", 100)),

                },
                "failures": [
                    {
                        "type": "timer",
                        "timing": 19.1,
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "udp_packet_flow_intermediate": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.iperf((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "separate_definitions": True,
                    "command_pre": ("measure_packet_flow", (
                        'h1', 'h6', '10.6.0.101', ["r1", "r2", "r3", "r5"], 30, 1, "udp_before_failure", [0, 20000],
                        "UDP Packet flow on routers before failure", "udp", 100)),
                    "command_post": ("measure_packet_flow", (
                        'h1', 'h6', '10.6.0.101', ["r1", "r2", "r3", "r5"], 30, 1, "udp_after_failure", [0, 22000],
                        "UDP Packet flow on routers after failure", "udp", 100)),

                },
                "failures": [
                    {
                        "type": "intermediate",
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            },
            "udp_packet_flow_concurrent": {
                "pre_execution": {
                    "use_pre_defined_function": False,
                    "command": lambda net: net.iperf((net['h1'], net['h6'])),
                },
                "source": "h1",
                "destination": "h6",
                "destination_ip": "10.6.0.101",
                "execute": {
                    "use_pre_defined_function": True,
                    "command": ("measure_packet_flow", (
                        'h1', 'h6', '10.6.0.101', ["r1", "r2", "r3", "r5"], 30, 1, "udp_concurrent_failure", [0, 22000],
                        "UDP Packet flow on routers before failure", "udp", 100)),

                },
                "failures": [
                    {
                        "type": "timer",
                        "timing": 19.1,
                        "execute": {
                            "use_pre_defined_function": True,
                            "command": (
                                "connection_shutdown", (['r3', 'r6'], ['Router 3', 'Router 6'], ['r3-eth2', 'r6-eth1']))
                        }
                    }
                ]
            }
        }
