import abc
from mininet.link import TCLink
from mininet.topo import Topo


class CustomTopo(Topo):
    limit = -1
    delay = -1
    host_count = 1
    
    def __init__(self, custom_limit=None, custom_delay=None):
        if custom_limit is not None:
            self.limit = custom_limit
        if custom_delay is not None:
            self.delay = custom_delay
        Topo.__init__(self)

    @abc.abstractmethod
    def get_policies(self):
        pass

    @abc.abstractmethod
    def get_routings(self):
        pass

    @abc.abstractmethod
    def get_tests(self):
        pass

    def get_policy_table_interface_map(self):
        table_interface_map = {}
        for router, policies in self.get_policies().items():
            table_interface_map[router] = {}
            for rule in policies["rules"]:
                if rule["table"] not in table_interface_map[router]:
                    table_interface_map[router][rule["table"]] = rule["inport"]

        return table_interface_map

    # Assumes that interfaces are named uniquely
    def get_interface_gateway_map(self):
        interface_gateway_map = {}
        for router, tables in self.get_routings().items():
            for table, routes in tables.items():
                for route in routes:
                    if route[2] not in interface_gateway_map:
                        interface_gateway_map[route[2]] = route[1]
        return interface_gateway_map

    # add limit to each link if limit was set
    def addLink(self, *args, **kwargs):
        if self.limit != -1 and "bw" not in kwargs:
            #kwargs["cls"] = TCLink
            kwargs["bw"] = self.limit
        #kwargs["max_queue_size"] = 1000
        if self.delay != -1 and "delay" not in kwargs:
            kwargs["delay"] = self.delay
        super().addLink(*args, **kwargs)

    #def addHost(self, *args, **kwargs):
        #kwargs["cpu"] = .5/self.host_count
     #   super().addHost(*args, **kwargs)
